@@ -240,7 +240,7 @@ emmeans = function(object, specs, by = NULL,
     }
     if (is.list(specs)) {
         return (emmeans.list(object, specs, by = by, 
-                             contr = contr, weights = weights))
+                             contr = contr, weights = weights, ...))
     }
     if (inherits(specs, "formula")) {
         spc = .parse.by.formula(specs)
@@ -270,6 +270,11 @@ emmeans = function(object, specs, by = NULL,
             warning("emmeans() results may be corrupted by removal of a nesting structure")
         }
         
+        # Ensure object is in standard order
+        ord = .std.order(RG@grid, RG@levels) ###do.call(order, unname(RG@grid[rev(names(RG@levels))]))
+        if(any(ord != seq_along(ord)))
+            RG = RG[ord]
+        
         if ((length(facs) == 1) && (facs == "1")) {  ### just want grand mean
             RG@levels[["1"]] = "overall"
             RG@grid[ ,"1"] = 1
@@ -422,8 +427,8 @@ emmeans = function(object, specs, by = NULL,
                  "To obtain a compact letter display, first call `emmeans()`,\n",
                  "then call `CLD()` on the result.")
         }
-        args = list() ###list(...)
-        ### args$data = NULL   # ensure 'data' not passed
+        args = list(...)
+        args$data = NULL   # ensure 'data' not passed
         args$object = result
         args$method = contr
         args$by = by
@@ -634,5 +639,14 @@ as.list.emmGrid = function(x, ...) {
     return(FALSE)
 }
 
-
+### Sort grid in standard order according to ordering of entries in levels. 
+### Thus .std.order(do.call(expand.grid, levels), levels) --> 1,2,...,nrow
+.std.order = function(grid, levels) {
+    tmp = lapply(rev(names(levels)), function(nm) {
+        x = grid[[nm]]
+        if (inherits(x, "factor")) as.integer(x)
+        else as.integer(factor(x, levels = levels[[nm]]))
+    })
+    do.call(order, tmp)
+}
 
