@@ -80,6 +80,14 @@ emm_basis.mlm = function(object, trms, xlev, grid, ...) {
     bas
 }
 
+#----------------------------------------------------------
+# manova objects
+recover_data.manova = function(object, ...) {
+    fcall = match.call(aov, object$call)   # need to borrow arg matching from aov()
+    recover_data(fcall, delete.response(terms(object)), object$na.action, ...)
+}
+
+
 
 
 #--------------------------------------------------------------
@@ -211,15 +219,27 @@ emm_basis.merMod = function(object, trms, xlev, grid, vcov.,
 #' @export
 recover_data.lme = function(object, data, ...) {
     fcall = object$call
-    if (!is.null(fcall$weights))
-        fcall$weights = nlme::varWeights(object$modelStruct)
+    if (!is.null(fcall$weights)) {  # painful -- we only get weights for complete cases
+        if (!is.null(object$na.action)) {
+            w = nlme::varWeights(object$modelStruct)
+            wts = rep(0, length(w) + length(object$na.action))
+            wts[-object$na.action] = w
+            fcall$weights = wts
+        }
+        else
+            fcall$weights = nlme::varWeights(object$modelStruct)
+    }
     recover_data(fcall, delete.response(object$terms), object$na.action, data = data, ...)
 }
 
 #' @export
 emm_basis.lme = function(object, trms, xlev, grid, 
-        mode = c("containment", "satterthwaite"), sigmaAdjust = TRUE, ...) {
+        mode = c("containment", "satterthwaite", "boot-satterthwaite", "auto"), sigmaAdjust = TRUE, ...) {
     mode = match.arg(mode)
+    if (mode == "auto")
+        mode = ifelse(is.null(object$apVar), "containment", "boot-satterthwaite")
+    if (is.null(object$apVar))
+        mode = "containment"
     contrasts = object$contrasts
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = contrasts)
@@ -233,8 +253,9 @@ emm_basis.lme = function(object, trms, xlev, grid,
     }
     nbasis = estimability::all.estble
     
-    if (mode == "satterthwaite") {
+    if (mode %in% c("satterthwaite", "boot-satterthwaite")) {
         G = try(gradV.kludge(object), silent = TRUE)
+        ###! not yet, doesn't work G = try(lme_grad(object, object$call, object$data, V))
         if (inherits(G, "try-error"))
             stop("Unable to estimate Satterthwaite parameters")
         dfargs = list(V = V, A = object$apVar, G = G)
@@ -256,7 +277,7 @@ emm_basis.lme = function(object, trms, xlev, grid,
         }
         dfargs = list(dfx = dfx)
     }
-    misc$initMesg = paste("Degrees-of-freedom method:", mode)
+    misc$initMesg = paste("d.f. method:", mode)
     list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
          dffun = dffun, dfargs = dfargs, misc = misc)
 }
@@ -266,19 +287,19 @@ emm_basis.lme = function(object, trms, xlev, grid,
 # model with a few random perturbations of y, then
 # regressing the changes in V against the changes in the 
 # covariance parameters
-gradV.kludge = function(object) {
+gradV.kludge = function(object, Vname = "varFix", call = object$call$fixed, data = object$data) {
     A = object$apVar
     theta = attr(A, "Pars")
-    V = object$varFix
+    V = object[[Vname]]
     sig = .01 * object$sigma
-    data = object$data
-    yname = all.vars(object$call$fixed)[1]
+    #data = object$data
+    yname = all.vars(call)[1]
     y = data[[yname]]
     n = length(y)
     dat = t(replicate(2 + length(theta), {
         data[[yname]] = y + sig * rnorm(n)
         mod = update(object, data = data)
-        c(attr(mod$apVar, "Pars") - theta, as.numeric(mod$varFix - V))
+        c(attr(mod$apVar, "Pars") - theta, as.numeric(mod[[Vname]] - V))
     }))
     dimnames(dat) = c(NULL, NULL)
     xcols = seq_along(theta)
@@ -287,8 +308,53 @@ gradV.kludge = function(object) {
     grad
 }
 
+# ### new way to get gradients for lme models
+# # (not ready for primetime...)
+# lme_grad = function(object, call, data, V) {
+#     obj = object$modelStruct
+#     conLin = object
+#     class(conLin) = class(obj)
+#     X = model.matrix(eval(call$fixed), data = data)
+#     y = data[[all.vars(call)[1]]]
+#     conLin$Xy = cbind(X, y)
+#     conLin$fixedSigma = FALSE
+#     grps = object$groups # May have to re-order these fancily?
+#     MEest = get("MEestimate", getNamespace("nlme")) ## workaround its not being exported
+#     func = function(x) {
+#         coef(obj) = x
+#         tmp = MEest(obj, grps, conLin)
+#         crossprod(tmp$sigma * tmp$varFix)
+#     }
+#     res = numDeriv::jacobian(func, coef(obj))
+#     G = lapply(seq_len(ncol(res)), function(j) matrix(res[, j], ncol = ncol(V)))
+#     G[[1 + length(G)]] = 2 * V  # gradient wrt log sigma
+#     G
+# }
+
+
 
 #--------------------------------------------------------------
+
+### new way to get gradients for gls models
+gls_grad = function(object, call, data, V) {
+    obj = object$modelStruct
+    conLin = object
+    class(conLin) = class(obj)
+    X = model.matrix(eval(call$model), data = data)
+    y = data[[all.vars(call)[1]]]
+    conLin$Xy = cbind(X, y)
+    conLin$fixedSigma = FALSE
+    func = function(x) {
+        obj = nlme::`coef<-`(obj, value = x)
+        tmp = nlme::glsEstimate(obj, conLin)
+        crossprod(tmp$sigma * tmp$varBeta)
+    }
+    res = numDeriv::jacobian(func, coef(obj))
+    G = lapply(seq_len(ncol(res)), function(j) matrix(res[, j], ncol = ncol(V)))
+    G[[1 + length(G)]] = 2 * V  # gradient wrt log sigma
+    G
+}
+
 ### gls objects (nlme package)
 recover_data.gls = function(object, ...) {
     fcall = object$call
@@ -298,16 +364,44 @@ recover_data.gls = function(object, ...) {
     recover_data(fcall, trms, object$na.action, ...)
 }
 
-emm_basis.gls = function(object, trms, xlev, grid, ...) {
+emm_basis.gls = function(object, trms, xlev, grid, 
+                         mode = c("auto", "df.error", "satterthwaite", "boot-satterthwaite"), ...) {
     contrasts = object$contrasts
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = contrasts)
     bhat = coef(object)
     V = .my.vcov(object, ...)
     nbasis = estimability::all.estble
-    dfargs = list(df = object$dims$N - object$dims$p)
-    dffun = function(k, dfargs) dfargs$df
-    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=list())
+    misc = list()
+    mode = match.arg(mode)
+    if (mode == "auto")
+        mode = ifelse(is.null(object$apVar), "df.error", "satterthwaite")
+    if (is.null(object$apVar))
+        mode = "df.error"
+    if (mode %in% c("satterthwaite", "boot-satterthwaite")) {
+        if (mode == "boot-satterthwaite") {
+            G = try(gradV.kludge(object, "varBeta", call = object$call,
+                                 data = eval(object$call$data)),
+                    silent = TRUE)
+        }
+        else
+            G = try(gls_grad(object, object$call, eval(object$call$data), V))
+        if (inherits(G, "try-error"))
+            stop("Unable to estimate Satterthwaite parameters")
+        dfargs = list(V = V, A = object$apVar, G = G)
+        dffun = function(k, dfargs) {
+            est = tcrossprod(crossprod(k, dfargs$V), k)
+            g = sapply(dfargs$G, function(M) tcrossprod(crossprod(k, M), k))
+            varest = tcrossprod(crossprod(g, dfargs$A), g)
+            2 * est^2 / varest
+        }
+    }
+    else {  ### mode == "df.error"
+        dfargs = list(df = object$dims$N - object$dims$p - length(attr(object$apVar, "Pars")))
+        dffun = function(k, dfargs) dfargs$df
+    }
+    misc$initMesg = paste("d.f. method:", mode)
+    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
 }
 
 
@@ -574,45 +668,33 @@ emm_basis.geese = function(object, trms, xlev, grid, vcov.method = "vbeta", ...)
 #--------------------------------------------------------------
 ### glmmADMB package
 
-recover_data.glmmadmb = recover_data.lm
-
-emm_basis.glmmadmb = function (object, trms, xlev, grid, ...) 
-{
-    contrasts = attr(model.matrix(object), "contrasts")
-    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
-    X = model.matrix(trms, m, contrasts.arg = contrasts)
-    bhat = glmmADMB::fixef(object)
-    V = .my.vcov(object, ...)
-    misc = list()
-    if (!is.null(object$family)) {
-        fam = object$family
-        misc$tran = object$link
-        misc$inv.lbl = "response"
-        if (!is.na(pmatch(fam,"binomial"))) 
-            misc$inv.lbl = "prob"
-        else if (!is.na(pmatch(fam,"poisson"))) 
-            misc$inv.lbl = "rate"
-    }
-    nbasis = estimability::all.estble
-    dffun = function(...) Inf
-    list(X = X, bhat = bhat, nbasis = nbasis, V = V, dffun = dffun, 
-         dfargs = list(), misc = misc)
-}
-
-
-# --------------------------------------------------------------
-### Explicit non-support for 'gam' objects (runs, but results are wrong)
-
-# emm_basis.gam = function(object, trms, xlev, grid, ...) {
-#     stop("Can't handle an object of class ", dQuote(class(object)[1]), "\n",
-#          .show_supported())
+# recover_data.glmmadmb = recover_data.lm
+# 
+# emm_basis.glmmadmb = function (object, trms, xlev, grid, ...) 
+# {
+#     contrasts = attr(model.matrix(object), "contrasts")
+#     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
+#     X = model.matrix(trms, m, contrasts.arg = contrasts)
+#     bhat = glmmADMB::fixef(object)
+#     V = .my.vcov(object, ...)
+#     misc = list()
+#     if (!is.null(object$family)) {
+#         fam = object$family
+#         misc$tran = object$link
+#         misc$inv.lbl = "response"
+#         if (!is.na(pmatch(fam,"binomial"))) 
+#             misc$inv.lbl = "prob"
+#         else if (!is.na(pmatch(fam,"poisson"))) 
+#             misc$inv.lbl = "rate"
+#     }
+#     nbasis = estimability::all.estble
+#     dffun = function(...) Inf
+#     list(X = X, bhat = bhat, nbasis = nbasis, V = V, dffun = dffun, 
+#          dfargs = list(), misc = misc)
 # }
 
 
 
-
-
-
 ### ----- Auxiliary routines -------------------------
 # Provide for vcov. argument in ref_grid call, which could be a function or a matrix
 
@@ -622,7 +704,7 @@ emm_basis.glmmadmb = function (object, trms, xlev, grid, ...)
 #' @export
 .my.vcov = function(object, vcov. = .statsvcov, ...) {
     if (is.function(vcov.))
-        vcov. = vcov.(object)
+        vcov. = vcov.(object, ...)
     else if (!is.matrix(vcov.))
         stop("vcov. must be a function or a square matrix")
     vcov.
