@@ -64,3 +64,17 @@ str(pigs.emm.ss)
 ## ------------------------------------------------------------------------
 emmeans(pigs.emm.ss, pairwise ~ type)
 
+## ------------------------------------------------------------------------
+warp <- transform(warpbreaks, treat = interaction(wool, tension))
+library(nlme)
+warp.gls <- gls(breaks ~ treat, weights = varIdent(form = ~ 1|treat), data = warp)
+( warp.emm <- emmeans(warp.gls, "treat") )
+
+## ------------------------------------------------------------------------
+warp.fac <- update(warp.emm, levels = list(
+                wool = c("A", "B"), tension = c("L", "M", "H")))
+str(warp.fac)
+
+## ------------------------------------------------------------------------
+contrast(warp.fac, "consec", by = "wool")
+
